/*
 * SMSPal - SMS machine gun
 *
 * Copyright (C) 2005 BLADOX, s.r.o.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <config.h>
#include <turbo/turbo.h>

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* *INDENT-OFF* */

static lc_char PROGMEM lc_Enter_Text[]={
        LC_EN("Enter Text:")
        LC_CZ("Text zpravy:")
        LC_END
};

/*
TODO
static lc_char PROGMEM lc_Send[]={
        LC_EN("Send?")
        LC_CZ("Poslat?")
        LC_END
};
*/

lc_char PROGMEM lc_Nr_Send[]={
        LC_EN("How many? ")
        LC_CZ("Kolik SMS? ")
        LC_END
};

lc_char PROGMEM lc_To_Many_ERR[]={
        LC_EN("Too many SMS sendings failed")
        LC_CZ("Prilis mnoho selhani odeslani SMS")
        LC_END
};

static lc_char PROGMEM lc_Sending[]={
        LC_EN("Sending ")
        LC_CZ("Posilam ")
        LC_END
};

static lc_char PROGMEM lc_Failed[]={
        LC_EN("Failed sending ")
        LC_CZ("Selhani ")
        LC_END
};

u8 PROGMEM t_SMSPal[] = "SMSPal";

/* *INDENT-ON* */

void smspal (Menu_selection_data * x)
{
  u8 *res;
  u8 *ms = NULL;
  u8 *text = NULL;
  u8 i;
  u8 len;
  u8 err = NO_ERROR;
  u8 *buf = buf_B ();
  u16 nr_to_send;
  u16 nr_send;
  u16 nr_err;
  u16 nr_err_raw;
  u8 *r;
  u8 *alpha;

  alpha = malloc (100);
  if (alpha == NULL)
  {
    err = ERR_NO_RAM;
    goto X_END;
  }

X_TEXT:
  res = get_input (locale (lc_Enter_Text), 0, 160, text, Q_GET_INPUT_ALPHABET | Q_GET_INPUT_SMS_PACKED);	//text STRING_SLV

  if (res == ENULL || res == NULL)
    goto X_END;

  len = res[0] - 1;

// @ fix for broken phones
  {
    u16 bits = len * 8;

    if (bits != 0 && bits % 7 == 0)
    {
      dbsp ("POSSIBLE @\n");
      if ((res[1 + len] & 0xfe) == 0)	//end @
      {
	dbsp ("POSSIBLE_1 @\n");
	res[1 + len] |= ('\r' << 1);	// @->CR
      }
    }
  }

  dbsp ("INPUT LEN ");
  dbch (len);
  dbc ('\n');

  if (text)
  {
    free (text);
    text = NULL;
  }

  text = malloc (len + 3);
  if (text == NULL)
  {
    err = ERR_NO_RAM;
    goto X_END;
  }

  text[0] = STRING_SLV;
  text[1] = res[1];
  text[2] = (len * 8) / 7;
  memcpy (&text[3], res + 2, len);

X_MSISDN:
  res = msisdn (NULL);

  if (res == ENULL)
    goto X_END;

  if (res == NULL)
    goto X_TEXT;

  ms = msisdncpy (res, MSISDN_ADN, MEM_R);

  if (ms == NULL)
  {
    err = ERR_NO_RAM;
    goto X_END;
  }

  res = get_input (locale (lc_Nr_Send), 1, 3, NULL, Q_GET_INPUT_DIGITS);
  if (res == ENULL)
    goto X_END;
  if (res == NULL)
  {
    free (ms);
    ms = NULL;
    goto X_MSISDN;
  }

  res[res[0] + 1] = '\0';
  nr_to_send = atoi (&res[2]);

  if (nr_to_send == 0)
    goto X_END;

  nr_send = 0;
  nr_err_raw = 0;
  nr_err = 0;

X_SEND:
  dbsp (">SEND_SMS\n");

  memcpy (buf, &text[3], len);

  r = alpha;
  r = sprints (r, locale (lc_Sending));
  r = sprinti (r, nr_send + 1);
  r = sprintc (r, '/');
  r = sprinti (r, nr_to_send);
  r = sprintc (r, '\n');

  r = sprints (r, locale (lc_Failed));
  r = sprinti (r, nr_err_raw);
  r = sprintc (r, '/');
  r = sprinti (r, nr_err);

  r = sprintc (r, '\0');

  res = send_sms (buf, len, ms, MSISDN_ADN, 0x00, 0x00, alpha, NULL);

  dbsp ("<SEND_SMS\n");

  i = get_tag (res, T_RESULT);
  if (i != 0)
  {
    i += 2;
    if (res[i] == 0x00)
    {
      nr_send++;
      nr_err_raw = 0;
      if (nr_send == nr_to_send)
	goto X_END;
    }
    else
    {
      nr_err_raw++;
      nr_err++;

#define MAX_ERR_RAW	10
      if (nr_err_raw == MAX_ERR_RAW)
      {
	display_text (locale (lc_To_Many_ERR), NULL);
	goto X_END;
      }
    }

    goto X_SEND;
  }

X_END:
  free (text);
  free (ms);
  free (alpha);

  if (err != NO_ERROR)
    perror (err);
}

void turbo_handler (u8 action, void *data)
{
  switch (action)
  {
    case ACTION_INSERT_MENU:
      insert_menu (t_SMSPal);
      break;
    case ACTION_MENU_SELECTION:
      stk_thread (smspal, NULL);
      break;
    default:
      break;
  }
}
